﻿package nicocache
{
    import flash.events.SyncEvent;
    import flash.net.SharedObject;
    import flash.net.ObjectEncoding;

    /**
     * 設定値管理クラス<br/>
     * 
     * できるだけ共有オブジェクトをラッピングする。
     */
    public class Config
    {
        /** インスタンス */
        private static var instance:Config = null;
        /** カラム用共有オブジェクト */
        private var _columnsSO:SharedObject;
        /** オプション */
        private var _optionSO:SharedObject;
        /** 自動振り分け設定 */
        private var _autoMoveSO:SharedObject;
        
        /** キー：削除確認オプション */
        public static const KEY_CONFIRM_OPTION:String = "deleteOption";
        /** キー：タイトル変更ダイアログ自動クローズ */
        public static const KEY_AUTO_CLOSE_TITLE_DIALOG:String = "autoCloseTitleDialog";
        /** キー：タイトル変更自動保存 */
        public static const KEY_AUTO_SAVE_TITLE_WHEN_AUTO_CLOSE:String = "autoSaveTitleWhenAutoClose";
        /** キー：再生リンクを常に新しいウィンドウで開く */
        public static const KEY_ALWAYS_OPEN_NEW_WINDOW:String = "alwaysOpenNewWindow";
        /** キー：ログリストを隠す */
        public static const KEY_HIDE_LOG_LIST:String = "hideLogList";
        /** キー：起動時にフォルダ一覧の再読み込みを行う */
        public static const KEY_ALWAYS_RELOAD_FOLDER_AT_STARTUP:String = "alowaysReloadFolderAtStartUp";
        /** キー：autoMoveRule.xmlを使用する */
        public static const KEY_USE_AUTO_MOVE_RULE_FILE:String =  "useAutoMoveRuleFile";
        /** キー：autoMoveRule.xmlのURL */
        public static const KEY_AUTO_MOVE_RULE_URL:String = "autoMoveRuleURL";
        /** キー：autoMoveRule */
        public static const KEY_AUTO_MOVE_RULE:String = "autoMoveRule";

        /**
         * コンストラクタ
         *
         * @param blocker ブロック
         */
        public function Config(blocker:Blocker)
        {
            if (blocker == null) throw new Error("getInstance()を使用してください。");

            // カラム情報
            _columnsSO = SharedObject.getLocal("columns");
            _columnsSO.objectEncoding = ObjectEncoding.AMF0;
            // オプション
            _optionSO = SharedObject.getLocal("option");
            _optionSO.objectEncoding = ObjectEncoding.AMF0;
            // 自動振り分け設定
            _autoMoveSO = SharedObject.getLocal("autoMove");
            _autoMoveSO.objectEncoding = ObjectEncoding.AMF0;
            
        }

        /**
         * インスタンスを取得します。
         *
         * @return インスタンス
         */
        public static function getInstance():Config 
        {
            if (instance == null) instance = new Config(new Blocker());
            return instance;
        }
        
        /**
         * 指定した共有オブジェクトからデータを取得します。
         * 
         * @param	so 共有オブジェクト
         * @param	key キー
         * @param	value デフォルト値
         * @return 結果
         */
        public static function get(so:SharedObject, key:String, value:*):*
        {
            if (so.data.hasOwnProperty(key)) return so.data[key];
            return value;
        }
        
        /**
         * ShareObject#setProperty(String, Object)で値をセットした後、フラッシュします。
         * 
         * @param	so 共有オブジェクト
         * @param	propertyName 共有オブジェクトのプロパティ名
         * @param	value プロパティの値（プロパティを削除する場合はnull）
         */
        public function setProperty(so:SharedObject, propertyName:String, value:Object=null):void
        {
            so.setProperty(propertyName, value);
            so.flush();
        }
        
        /** 確認オプションプロパティ */
        public function get confirmOption():int { 
            return int(get(_optionSO, KEY_CONFIRM_OPTION, 0));
        }
        /** 確認オプションプロパティ */
        public function set confirmOption(value:int):void
        {
            setProperty(_optionSO, KEY_CONFIRM_OPTION, value);
        }
        [Bindable]
        /** タイトル変更ダイアログ自動クローズ */
        public function get titleChangeDialogAutoClose():Boolean {
            return Boolean(get(_optionSO, KEY_AUTO_CLOSE_TITLE_DIALOG, false));
        }
        /** タイトル変更ダイアログ自動クローズ */
        public function set titleChangeDialogAutoClose(value:Boolean):void {
            setProperty(_optionSO, KEY_AUTO_CLOSE_TITLE_DIALOG, value);
        }
        [Bindable]
        /** タイトル変更自動保存 */
        public function get titleChangeAutoSave():Boolean {
            return get(_optionSO, KEY_AUTO_SAVE_TITLE_WHEN_AUTO_CLOSE, false) as Boolean;
        }
        /** タイトル変更自動保存 */
        public function set titleChangeAutoSave(value:Boolean):void {
            setProperty(_optionSO, KEY_AUTO_SAVE_TITLE_WHEN_AUTO_CLOSE, value);
        }
        [Bindable]
        /** 再生リンクを常に新しいウィンドウで開く */
        public function get alwaysOpenNewWindow():Boolean {
            return Boolean(get(_optionSO, KEY_ALWAYS_OPEN_NEW_WINDOW, false));
        }
        /** 再生リンクを常に新しいウィンドウで開く */
        public function set alwaysOpenNewWindow(value:Boolean):void {
            setProperty(_optionSO, KEY_ALWAYS_OPEN_NEW_WINDOW, value);
        }
        [Bindable]
        /** ログリストを隠す */
        public function get hideLogList():Boolean {
            return Boolean(get(_optionSO, KEY_HIDE_LOG_LIST, false));
        }
        /** ログリストを隠す */
        public function set hideLogList(value:Boolean):void {
            setProperty(_optionSO, KEY_HIDE_LOG_LIST, value);
        }
        [Bindable]
        /** 起動時にフォルダ一覧の再読み込みを行う */
        public function get alwaysReloadFolderAtStartUp():Boolean {
            return Boolean(get(_optionSO, KEY_ALWAYS_RELOAD_FOLDER_AT_STARTUP, true));
        }
        /** 起動時にフォルダ一覧の再読み込みを行う */
        public function set alwaysReloadFolderAtStartUp(value:Boolean):void {
            setProperty(_optionSO, KEY_ALWAYS_RELOAD_FOLDER_AT_STARTUP, value);
        }
        [Bindable]
        /** autoMoveRule.xmlを使用する */
        public function get useAutoMoveRuleFile():Boolean {
            return Boolean(get(_autoMoveSO, KEY_USE_AUTO_MOVE_RULE_FILE, true));
        }
        /** autoMoveRule.xmlを使用する */
        public function set useAutoMoveRuleFile(value:Boolean):void {
            setProperty(_autoMoveSO, KEY_USE_AUTO_MOVE_RULE_FILE, value);
        }
        [Bindable]
        /** autoMoveRule.xmlのURL */
        public function get autoMoveRuleURL():String {
            return String(get(_autoMoveSO, KEY_AUTO_MOVE_RULE_URL, 
                    "http://www.nicovideo.jp/local/resource/settings/autoMoveRule.xml"));
        }
        /** autoMoveRule.xmlのURL */
        public function set autoMoveRuleURL(value:String):void {
            setProperty(_autoMoveSO, KEY_AUTO_MOVE_RULE_URL, value);
        }
        /** autoMoveRule */
        public function get autoMoveRule():XML {
            return XML(get(_autoMoveSO, KEY_AUTO_MOVE_RULE, <rules></rules>));
        }
        /** autoMoveRule */
        public function set autoMoveRule(value:XML):void {
            _autoMoveSO.setProperty(KEY_AUTO_MOVE_RULE, value.toXMLString());
        }
        
        /** カラム用共有オブジェクト */
        public function get columns():SharedObject
        {
            return _columnsSO;
        }
        
    }
}

/**
 * ブロッククラス
 *
 */
class Blocker
{
}

