﻿package nicocache.action 
{
    import mx.core.mx_internal;
    import flash.display.Sprite;
    import mx.controls.Alert;
    import mx.core.Application;
    import mx.events.CloseEvent;
    import mx.rpc.AsyncDispatcher;
    import mx.rpc.AsyncToken;
    import mx.rpc.Fault;
    import mx.rpc.http.HTTPService;
    import mx.rpc.events.ResultEvent;
    import mx.rpc.events.FaultEvent;
    import nicocache.folder.FolderInfo;
    import nicocache.Config;
    import nicocache.Constant;
    
    import nicocache.log.Logger;
    import nicocache.utils.NetUtil;
    
    use namespace mx_internal;
    
    /**
     * 削除用HTTPServiceクラス<br/>
     * <pre>
     *   例：
     *     var deleteAction:DeleteAction = new DeleteAction(...);
     *     deleteAction.setDeaultHandler();
     *     deleteAction.execute();
     * </pre>
     */
    public class DeleteAction extends HTTPService
    {
        /** アクションタイプ */
        public var type:String = Constant.ACTION_TYPE_SINGLE;
        /** 動画ID */
        public var id:String;
        /** フォルダパス */
        public var folderPath:String;
        /** 一時ファイルフラグ */
        public var tempFlag:Boolean;
        /** エコノミーフラグ */
        public var economyFlag:Boolean;
        /** パラメータ */
        public var parameters:Object = null;
        
        /**
         * コンストラクタ
         * @param	id 動画ID
         * @param	folderPath フォルダパス
         * @param	tempFlag 一時ファイルフラグ
         * @param	economyFlag エコノミーフラグ
         */
        public function DeleteAction(id:String, folderPath:String, tempFlag:Boolean, economyFlag:Boolean)
        {
            super();
            
            this.id = id;
            this.folderPath = folderPath;
            this.tempFlag = tempFlag;
            this.economyFlag = economyFlag;
            
            // 設定
            url = NetUtil.getDeleteURL(id, tempFlag, economyFlag);
            resultFormat = HTTPService.RESULT_FORMAT_TEXT;
            contentType = "application/x-www-form-urlencoded; charset=UTF-8";
            // 優先度高で登録
            addEventListener(ResultEvent.RESULT, resultEventHandler, false, int.MAX_VALUE);
        }
        
        /**
         * 削除を開始します。
         */
        public function execute():AsyncToken
        {
            var token:AsyncToken = null;
            
            if (type == Constant.ACTION_TYPE_SINGLE)
            {
                var conf:Config = Config.getInstance();
                // 確認する場合
                if (conf.confirmOption == Constant.CONFIRM_OPTION_ALWAYS)
                {
                    showConfirmDialog(function(closeEvent:CloseEvent):void {
                        // YESボタンが押されたら削除
                        if (closeEvent.detail == Alert.YES) token = send(parameters); // 削除実行
                    });
                }
                // 確認しない場合
                else token = send(parameters);
            }
            else token = send(parameters);
            
            return token;
        }
        
        /**
         * 確認ダイアログを表示します。
         * @param	closeHandler ダイアログを閉じた時の動作を実装した関数
         */
        public function showConfirmDialog(closeHandler:Function=null):void
        {
            var flags:uint = Alert.YES | Alert.NO;
            
            if (type == Constant.ACTION_TYPE_MULTI) flags |= Alert.CANCEL;
            
            Alert.show("キャッシュ（" + id + "）を削除しますか？", "キャッシュの削除確認", 
                flags, Application.application as Sprite, closeHandler);
        }
        
        /**
         * NicoCache_nlの動画削除APIにメッセージを送信します。
         * 
         * @param	parameters パラメータ
         * @return トークン
         */
        override public function send(parameters:Object = null):AsyncToken {
            var token:AsyncToken = super.send(parameters);
            
            token.url = url;
            token.id = id;
            token.folderPath = folderPath;
            token.economyFlag = economyFlag;
            token.tempFlag = tempFlag;
            
            return token;
        }
        
        /**
         * デフォルトのハンドラを優先度100で登録します。
         */
        public function setDefaultHandler():void
        {
            // 優先度100で登録
            addEventListener(ResultEvent.RESULT, defaultResultHandler, false, 100);
            addEventListener(FaultEvent.FAULT, defaultFaultHandler, false, 100);
        }
        
        /**
         * 結果のメッセージが「OK」でない場合、FaultEventを送出するハンドラ。
         * @param	event イベントオブジェクト
         */
        protected function resultEventHandler(event:ResultEvent):void
        {
            var result:String = String(event.result);
            if (result.indexOf("OK") < 0)
            {
                var fault:Fault = new Fault("DELETE_ERROR", "キャッシュの削除に失敗しました。");
                var faultEvent:FaultEvent = FaultEvent.createEvent(fault, event.token);
                new AsyncDispatcher(dispatchRpcEvent, [faultEvent], 10);  // internal
                event.stopImmediatePropagation();
            }
        }
        
        /**
         * 削除成功時のデフォルトハンドラです。<br/>
         * setDefaultHandler()メソッドを呼ぶと登録されます。
         * 
         * @param	event イベントオブジェクト
         */
        protected function defaultResultHandler(event:ResultEvent):void
        {
            Logger.info("削除成功 ⇒ " + event.token.url);
            
            var folderInfo:FolderInfo = FolderInfo.getInstance();
            // 対象フォルダが現在表示しているフォルダの場合
            if (folderInfo.path == event.token.folderPath)
            {
                var app:Object = Application.application;
                if (app.hasOwnProperty("removeListData"))
                    app.removeListData(event.token.id);
            }
        }
        
        /**
         * 削除失敗時のデフォルトハンドラです。<br/>
         * setDefaultHandler()メソッドを呼ぶと登録されます。
         * 
         * @param	event イベントオブジェクト
         */
        protected function defaultFaultHandler(event:FaultEvent):void
        {
            Logger.error("削除失敗 ⇒ " + event.token.url);
        }
    }
    
}