﻿package nicocache.controls.columnlist
{
    import flash.net.SharedObject;
    import nicocache.Config;
    import nicocache.Constant;
    import nicocache.utils.BooleanUtil;
    import nicocache.controls.column.*;
    
    public class DefaultColumnList
    {
        private var _view:String;
        private var _path:String;
        protected var _columns:Array = new Array();
        protected var _columnMap:Object = new Object();
        protected var conf:Config = Config.getInstance();
        
        private static const SEP:String = "_";
        
        /**
         * コンストラクタ
         * 
         * @param	view フォルダのview属性
         * @param	path フォルダのパス
         */
        public function DefaultColumnList(view:String, path:String)
        {
            _view = view;
            _path = path;
            createDefaultColumns();
        }
        
        /** ID */
        public function get id():String
        {
            return view + SEP + path;
        }
        
        /** カラムリストのタイプ */
        public function get view():String
        {
            return _view;
        }
        
        /** フォルダのパス */
        public function get path():String
        {
            return _path;
        }

        /**
         * デフォルトのカラムを作成します。
         * 
         * @return カラム配列
         */
        private function createDefaultColumns():void
        {
            addColumn(new CheckBoxColumn());
            addColumn(new IdColumn());
            addColumn(new TitleColumn());
            addColumn(new FolderNameColumn());
            addColumn(new PathColumn());
            addColumn(new SizeColumn());
            addColumn(new LastModifyColumn());
            addColumn(new FormatColumn());
        }
        
        /**
         * カラムを追加します。
         * @param	col カラム
         */
        public function addColumn(col:CustomColumn):void
        {
            var so:SharedObject = conf.columns;
            
            col.columnId = id + SEP + col.dataField;
            col.width = Config.get(so, col.columnId + Constant.COL_ATTR_WIDTH, col.width);
            col.visible = BooleanUtil.parseBoolean(Config.get(so, view + col.dataField + Constant.COL_ATTR_VISIBLE, col.visible));
            _columns.push(col);
            _columnMap[col.dataField] = col;
        }
        
        /**
         * データフィールドからカラムを取得します。
         * 
         * @param	value データフィールド
         * @return カラムオブジェクト
         */
        public function getColumn(value:String):CustomColumn
        {
            return _columnMap[value] as CustomColumn;
        }
        
        /**
         * カラム一覧を配列で取得します。
         */
        public function get columns():Array
        {
            var _arr:Array = _columns.slice(0);
            var so:SharedObject = conf.columns;
            var key:String;
            
            for each(var col:CustomColumn in _arr)
            {
                key = view + col.dataField + Constant.COL_ATTR_VISIBLE;
                col.visible = Config.get(so, key, col.visible);
                var value:Number = Config.get(so, col.columnId, col.width) as Number;
                col.setWidthInternal(value);
            }
            return _arr;
        }
    }
}

