﻿package nicocache.log
{
    import mx.controls.List;
    import nicocache.log.LoggerHandler;
    import nicocache.log.Logger;
    import mx.collections.ArrayCollection;
    import mx.collections.ICollectionView;
    import mx.collections.IViewCursor;
    import mx.collections.CursorBookmark;
    import mx.utils.StringUtil;
    import mx.formatters.DateFormatter;

    public class ListLoggerHandler implements LoggerHandler
    {
        private var list:List = null;
        public var maxSize:int = 100;
        private var formatter:DateFormatter = new DateFormatter();
        
        /**
         * メッセージをリストに追加します。
         * 
         * @param	lvl レベル
         * @param	msg メッセージ
         */
        private function addMessage(lvl:int, msg:String):void
        {
            if (list == null) return;
            
            var dt:Date = new Date();
            var source:ICollectionView = ICollectionView(list.dataProvider);
            var cursor:IViewCursor = source.createCursor();
            formatter.formatString = "YYYY/MM/DD JJ:NN:SS"

            if (source.length >= maxSize)
            {
                cursor.remove();
            }

            cursor.seek(CursorBookmark.LAST, 1);
            
            cursor.insert(StringUtil.substitute("{0} [{1}]: {2}", 
                formatter.format(dt), Logger.prefix[lvl], msg));
            list.scrollToIndex(source.length);
        }
        
        /**
         * メッセージを構築します。
         * 
         * @param	s 文字列。可変引数を使う場合は{n}を使用してください。nは0から始まる整数。
         * @param	... args 可変引数
         * @return メッセージ
         */
        private function getMessage(s:String, ... args):String
        {
            if (args[0][0].length == 0) return s;
            
            return StringUtil.substitute(s, args[0][0]);
        }

        public function setList(target:List):void
        {
            list = target;
        }

        /**
         * デバッグログを表示します。
         * 
         * @param	message メッセージ
         * @param	... args 可変引数
         */
        public function debug(message:String, ... args):void
        {
            addMessage(Logger.DEBUG, getMessage(message, args));
        }

        /**
         * 情報ログを表示します。
         * 
         * @param	message メッセージ
         * @param	... args 可変引数
         */
        public function info(message:String, ... args):void
        {
            addMessage(Logger.INFO, getMessage(message, args));
        }

        /**
         * 警告ログを表示します。
         * 
         * @param	message メッセージ
         * @param	... args 可変引数
         */
        public function warn(message:String, ... args):void
        {
            addMessage(Logger.WARN, getMessage(message, args));
        }

        /**
         * エラーログを表示します。
         * 
         * @param	message メッセージ
         * @param	... args 可変引数
         */
        public function error(message:String, ... args):void
        {
            addMessage(Logger.ERROR, getMessage(message, args));
        }
    }
}

