﻿package nicocache.utils
{
    import flash.net.navigateToURL;
    import flash.net.URLRequest;
    import nicocache.log.Logger;

    public class NetUtil
    {
        /**
         * 指定したURLへ移動します。
         * 
         * @param	url URL
         * @param	window ウィンドウターゲット
         */
        public static function windowOpen(url:String, window:String="_self"):void
        {
            navigateToURL(new URLRequest(url), window);
        }
        
        /**
         * ファイルダウンロード用のURLを取得します。
         *
         * @param id キャッシュID
         * @param format 拡張子
         * @return URL
         */
        public static function getDownloadURL(id:String, format:String):String
        {
            if (/^\w{2}\d+(low)?$/.exec(id)) {
                return "http://www.nicovideo.jp/cache/" + id + "." + format;
            } else {
                return "http://www.nicovideo.jp/cache/" + id;
            }
        }

        /**
         * 音声ファイル抽出用のURLを取得します。
         *
         * @param id キャッシュID
         * @param format 拡張子
         * @return URL
         */
        public static function getExtractAudioURL(id:String, format:String):String
        {
            if (format && format == "swf")
            {
                Logger.warn(id + " ⇒ Flashムービーファイル(*.swf)には対応していません");
                return null;
            }
            var ext:String = ".mp3";
            if (format == "mp4") ext = ".m4a";
            
            return "http://www.nicovideo.jp/cache/" + id + ext;
        }

        /**
         * 動画削除用のURLを取得します。
         *
         * @param id キャッシュID
         * @param temp 一時ファイルフラグ
         * @param exonomy エコノミーフラグ
         * @return URL
         */
        public static function getDeleteURL(id:String, temp:Boolean, economy:Boolean):String
        {
            return "http://www.nicovideo.jp/cache/ajax_rm" + (temp ? "tmp?" : "?") + id;
        }

        /**
         * 動画移動用のURLを取得します。
         *
         * @param id キャッシュID
         * @param newFolder 移動先フォルダ名
         * @return URL
         */
        public static function getMoveURL(id:String, newFolder:String):String
        {
            return "http://www.nicovideo.jp/cache/ajax_move?" + id + "-" + encodeURIComponent(newFolder);
        }
        
        /**
         * NicoCache_nlのタイトル変更APIのURLを取得します。
         * 
         * @param	id キャッシュID
         * @param	newTitle 新しいタイトル
         * @param	economy エコノミーフラグ
         * @return URL
         */
        public static function getRenameTitleURL(id:String, newTitle:String, economy:Boolean):String
        {
            return "http://www.nicovideo.jp/cache/ajax_title?" + id + 
                    (newTitle == "" ? "" : "-") + encodeURIComponent(newTitle);
        }
        
        /**
         * タグ検索URLを取得します。
         * 
         * @param	tag タグ
         * @return URL
         */
        public static function getTagSearchURL(tag:String):String
        {
            return "http://www.nicovideo.jp/tag/" + encodeURIComponent(tag);
        }
        
        /**
         * 削除済み動画のサムネイルURLを取得します。
         * 
         * @return URL
         */
        public static function getDeleteImageURL():String
        {
            return "http://www.nicovideo.jp/img/thumb/del_img.jpg";
        }
        
        //---------------------------------------------------------------------
        // ニコニコ動画のAPI
        //---------------------------------------------------------------------
        
        /**
         * flv取得URL、コメントXML取得URLを取得します。
         * @param	id 動画IDまたはスレッドID
         * @return URL
         */
        public static function getApiGetFlv(id:String):String
        {
            return "http://www.nicovideo.jp/api/getflv/" + id;
        }
        
        /**
         * サムネイル情報取得URLを取得します。
         * @param	id 動画IDまたはスレッドID
         * @return URL
         */
        public static function getApiGetThumbinfo(id:String):String
        {
            return "http://www.nicovideo.jp/api/getthumbinfo/" + id;
        }
    }
}
